package net.locusworks.hashvalidator;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

import org.apache.commons.codec.digest.DigestUtils;

import com.beust.jcommander.JCommander;

import net.locusworks.hashvalidator.enums.HashType;

public class Entry {

  public static void main(String[] argv) throws IOException {
    Args args = new Args();
    JCommander commander = JCommander.newBuilder().addObject(args).programName("Hash Validator").build();
    try {
      commander.parse(argv);
    } catch (Exception ex) {
      System.err.println(ex.getMessage());
      commander.usage();
      System.exit(-1);
    }
    
    if (args.getString() != null && !args.getString().trim().isEmpty()) {
      String val = args.getString().trim();
      String hash = "";
      switch (args.getHashType()) {
        case MD5:
          hash = DigestUtils.md5Hex(val);
          break;
        case SHA1:
          hash = DigestUtils.sha1Hex(val);
          break;
        case SHA512:
          hash = DigestUtils.sha512Hex(val);
          break;
        default:
          break;
      }
      System.out.println(String.format("The %s hash for %s is %s", args.getHashType(), val, hash));
    }
    
    if (args.getFiles() == null || args.getFiles().size() == 0) return;
    
    if(args.getHashes() != null && (args.getHashes().size() != args.getFiles().size()))
        throw new IllegalArgumentException("The number of entered hashes do not match the number of files entered");
    
    if (args.getHashes() != null) {
      for (int i = 0; i < args.getFiles().size(); i++) {
        Path file = args.getFiles().get(i);
        String expectedHash = args.getHashes().get(i).trim();
        String actualHash = getHash(file, args.getHashType());
        if (actualHash.equalsIgnoreCase(expectedHash)) {
          System.out.println(String.format("%s matches the %s hash of %s", file.getFileName(), args.getHashType(), actualHash));
        } else {
          System.out.println(String.format("%s %s hash do not match. Expected: %s -> Actual: %s", 
              file.getFileName(), args.getHashType(), expectedHash, actualHash));
        }
      }
      return;
    } 
    
    for(Path file : args.getFiles()) {
      String hash = getHash(file, args.getHashType());
      System.out.println(String.format("%s %s hash is %s", file.getFileName(), args.getHashType(), hash));
    }

  }

  private static String getHash(Path file, HashType hashType) throws IOException {
    String hash = null;
    switch(hashType) {
      case MD5:
        hash = DigestUtils.md5Hex(Files.newInputStream(file));
        break;
      case SHA1:
        hash = DigestUtils.sha1Hex(Files.newInputStream(file));
        break;
      case SHA512:
        hash = DigestUtils.sha512Hex(Files.newInputStream(file));
        break;
      default:
        break;
    }
    return hash;
  }

}
